#ifndef LCD_H
#define LCD_H

/*************************************************************************
 Title	:   C include file for the HD44780U LCD library (lcd.c)
 Author:    Peter Fleury <pfleury@gmx.ch>  http://jump.to/fleury
 File:	    $Id: lcd.h,v 1.13.2.2 2006/01/30 19:51:33 peter Exp $
 Software:  AVR-GCC 3.3
 Hardware:  modifed for any Xmega device by Pavel Kharchenko
***************************************************************************/

#if (__GNUC__ * 100 + __GNUC_MINOR__) < 303
#error "This library requires AVR-GCC 3.3 or later, update to newer AVR-GCC compiler !"
#endif

#include <inttypes.h>
#include <avr/pgmspace.h>
#define F_CPU 2000000		// AVR clock frequency in Hz, used by util/delay.h
//#if F_CPU==0  /**< clock frequency in Hz, used to calculate delay timer */
//#error "ERROR! define CPU frequence!"
//#endif 

#define LCD_CONTROLLER_KS0073 0  /**< Use 0 for HD44780 controller, 1 for KS0073 controller */
#define LCD_LINES             2     /**< number of visible lines of the display */
#define LCD_DISP_LENGTH      16     /**< visibles characters per line of the display */
#define LCD_LINE_LENGTH    40//0x40     /**< internal line length of the display    */
#define LCD_START_LINE1    0x00     /**< DDRAM address of first char of line 1 */
#define LCD_START_LINE2    0x40     /**< DDRAM address of first char of line 2 */
#define LCD_START_LINE3    0x10     /**< DDRAM address of first char of line 3 */
#define LCD_START_LINE4    0x50     /**< DDRAM address of first char of line 4 */
#define LCD_WRAP_LINES        0     /**< 0: no wrap, 1: wrap at end of visibile line */

#define LCD_DATA0_PORT   PORTC     	/**< port for 4bit data bit 0 */
#define LCD_DATA0_PIN    PIN3_bm           	/**< pin for 4bit data bit 0  */
#define LCD_DATA1_PORT   PORTC     	/**< port for 4bit data bit 1 */
#define LCD_DATA1_PIN    PIN4_bm            	/**< pin for 4bit data bit 1  */
#define LCD_DATA2_PORT   PORTC     	/**< port for 4bit data bit 2 */
#define LCD_DATA2_PIN    PIN5_bm            	/**< pin for 4bit data bit 2  */
#define LCD_DATA3_PORT   PORTC     	/**< port for 4bit data bit 3 */
#define LCD_DATA3_PIN    PIN6_bm            	/**< pin for 4bit data bit 3  */
#define LCD_RS_PORT      PORTC     	/**< port for RS line         */
#define LCD_RS_PIN       PIN0_bm            	/**< pin  for RS line         */
#define LCD_RW_PORT      PORTC     	/**< port for RW line         */
#define LCD_RW_PIN       PIN1_bm            	/**< pin  for RW line         */
#define LCD_E_PORT       PORTC     	/**< port for Enable line     */
#define LCD_E_PIN        PIN2_bm            	/**< pin  for Enable line     */

/* instruction register bit positions, see HD44780U data sheet */
#define LCD_CLR               0      /* DB0: clear display                  */
#define LCD_HOME              1      /* DB1: return to home position        */
#define LCD_ENTRY_MODE        2      /* DB2: set entry mode                 */
#define LCD_ENTRY_INC         1      /* DB1: 1=increment, 0=decrement     */
#define LCD_ENTRY_SHIFT       0      /* DB2: 1=display shift on           */
#define LCD_ON                3      /* DB3: turn lcd/cursor on             */
#define LCD_ON_DISPLAY        2      /* DB2: turn display on              */
#define LCD_ON_CURSOR         1      /* DB1: turn cursor on               */
#define LCD_ON_BLINK          0      /* DB0: blinking cursor ?          */
#define LCD_MOVE              4      /* DB4: move cursor/display            */
#define LCD_MOVE_DISP         3      /* DB3: move display (0-> cursor) ?  */
#define LCD_MOVE_RIGHT        2      /* DB2: move right (0-> left) ?      */
#define LCD_FUNCTION          5      /* DB5: function set                   */
#define LCD_FUNCTION_2LINES   3      /* DB3: two lines (0->one line)      */
#define LCD_FUNCTION_10DOTS   2      /* DB2: 5x10 font (0->5x7 font)      */
#define LCD_CGRAM             6      /* DB6: set CG RAM address             */
#define LCD_DDRAM             7      /* DB7: set DD RAM address             */
#define LCD_BUSY              7      /* DB7: LCD is busy                    */

/* set entry mode: display shift on/off, dec/inc cursor move direction */
#define LCD_ENTRY_DEC            0x04   /* display shift off, dec cursor move dir */
#define LCD_ENTRY_DEC_SHIFT      0x05   /* display shift on,  dec cursor move dir */
#define LCD_ENTRY_INC_           0x06   /* display shift off, inc cursor move dir */
#define LCD_ENTRY_INC_SHIFT      0x07   /* display shift on,  inc cursor move dir */

/* display on/off, cursor on/off, blinking char at cursor position */
#define LCD_DISP_OFF             0x08   /* display off                            */
#define LCD_DISP_ON              0x0C   /* display on, cursor off                 */
#define LCD_DISP_ON_BLINK        0x0D   /* display on, cursor off, blink char     */
#define LCD_DISP_ON_CURSOR       0x0E   /* display on, cursor on                  */
#define LCD_DISP_ON_CURSOR_BLINK 0x0F   /* display on, cursor on, blink char      */

/* move cursor/shift display */
#define LCD_MOVE_CURSOR_LEFT     0x10   /* move cursor left  (decrement)          */
#define LCD_MOVE_CURSOR_RIGHT    0x14   /* move cursor right (increment)          */
#define LCD_MOVE_DISP_LEFT       0x18   /* shift display left                     */
#define LCD_MOVE_DISP_RIGHT      0x1C   /* shift display right                    */

/* function set: set interface data length and number of display lines */
#define LCD_FUNCTION_4BIT_1LINE  0x20   /* 4-bit interface, single line, 5x7 dots */
#define LCD_FUNCTION_4BIT_2LINES 0x28   /* 4-bit interface, dual line,   5x7 dots */

#define LCD_MODE_DEFAULT     ((1<<LCD_ENTRY_MODE) | (1<<LCD_ENTRY_INC) )

extern void lcd_init(uint8_t dispAttr);
extern void lcd_clrscr(void);
extern void lcd_home(void);
extern void lcd_gotoxy(uint8_t x, uint8_t y);
extern void lcd_putc(char c);
extern void lcd_puts(const char *s);
extern void lcd_puts_p(const char *progmem_s);
extern void lcd_command(uint8_t cmd);
extern void lcd_data(uint8_t data);
//extern void lcd_delay(void);

#define lcd_puts_P(__s)         lcd_puts_p(PSTR(__s))

#endif //LCD_H
